# @summary Installs the CIS Assessor along with required dependencies.
#
# CIS CAT Pro scanner requires a JRE as a prerequisite.
#
# @param allow_insecure
#   Enables skipping of certificate validation when the scanner_source parameter is specified.
#
#
#   This parameter should be used only with the scanner_source parameter.
#
# @param linux_manage_unzip
#   Determines if this module should manage the installation of unzip on Linux
#
# @param manage_java
#   Determines if this module should manage the installation of java
#   Note: This param is DEPRECATED and will be removed in a subsequent version.
#
# @param scanner_source
#   The URL to the location of a specific Assessor distribution within comply
#   This may be specified as a source attribute of a [file resource](https://puppet.com/docs/puppet/latest/types/file.html#file-attribute-source)
#
#   In the Puppet Comply application config, if you specified "We want to access the assessor through our ingress controller",
#   then set this value to https://`PE TLS FQDN`/assessor, otherwise set this value to https://`Comply Hostname`:30303/assessor
#
# @param scanner_source_linux
#   The URL to the location of a specific linux bundle should the user wish to host outside of comply.
#
# @param scanner_source_mac
#   The URL to the location of a specific mac bundle should the user wish to host outside of comply.
#
# @param scanner_source_windows
#   The URL to the location of a specific windows bundle should the user wish to host outside of comply.
#
# @param scanner_checksum_linux
#   Specify the sha256 checksum of the Linux Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_checksum_mac
#   Specify the sha256 checksum of the Mac Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_checksum_windows
#   Specify the sha256 checksum of the Windows Assessor archive provided in `scanner_source`.
#   Defaults to the checksum of the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param scanner_version
#   Specify the version of the Assessor archive provided in `scanner_source`.
#   Defaults to the respective version of the scanner as part of Comply deployed in Puppet Application manager.
#
# @param use_mtls
#   Use mutual TLS authentication. Comply is configured to only accept requests from nodes with valid certificates issued with the same
#   CA cert configured in Replicated. Ensure you have configured Comply properly using certs generated on your PE instance.
#
# @param windows_manage_wget
#   Allow Comply to manage the wget package. Required for fetching the Assessor.
#
# @param assessor_download_token
#   A bearer token to be used when downloading the Assessor.
#
# @param limits
#   Apply limits to the Assessor process, for example limit CPU usage.
#   The limits parameter is a hash with the following structure:
#   {
#     "systemd-run" => { # Linux only if /usr/bin/systemd-run is available
#       "CPUQuota" => 50 # Percentage of a single CPU core that the Assessor process is allowed to use, can be >100 for multiple cores.
#     },
#     "nice" => { # Linux and Mac only
#       "increment" => 10 # Increment the nice value of the Assessor process, can be negative.
#     }
#   }
#
# @example Manage packages elsewhere on Linux
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://comply.10.10.10.10.nip.io/assessor',
#     linux_manage_unzip => false,
#   }
#
# @example Install the CIS Assessor when Comply is configured to access the assessor through an ingress controller
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://pe-tls-fqdn.10.10.10.10.nip.io/assessor',
#     linux_manage_unzip => false,
#   }
#
# @example Install the CIS Assessor when Comply is configured to access the assessor through a NodePort
#   class { 'comply::scanners::ciscat':
#     scanner_source     => 'https://comply.10.10.10.10.nip.io:30303/assessor',
#     linux_manage_unzip => false,
#   }
#
# @example Install the CIS Assessor when hosting at a user provided location
#   class { 'comply::scanners::ciscat':
#     scanner_source_linux     => https://server.puppet.com/Assessor-CLI-windows-v4.26.0.zip',
#     scanner_source_windows   => https://server.puppet.com/Assessor-CLI-linux-v4.26.0.zip',
#   }
#
# @example Limit the Assessor process to 50% of a single CPU core on Linux.
#   class { 'comply::scanners::ciscat':
#     scanner_source => 'https://comply.10.10.10.10.nip.io/assessor',
#     limits         => {"systemd-run" => {"CPUQuota" => 50}},
#   }
#
# @example Increment the nice value of the Assessor process on Linux or macOS.
#   class { 'comply::scanners::ciscat':
#     scanner_source => 'https://comply.10.10.10.10.nip.io/assessor',
#     limits         => {"nice" => {"increment" => 10}},
#   }
#
class comply::scanners::ciscat (
  Boolean $allow_insecure                                    = $comply::allow_insecure,
  Boolean $linux_manage_unzip                                = $comply::linux_manage_unzip,
  Boolean $win_manage_7zip                                   = $comply::win_manage_7zip,
  Boolean $manage_chocolatey                                 = $comply::manage_chocolatey,
  Boolean $manage_java                                       = $comply::manage_java,
  Optional[Stdlib::Filesource] $scanner_source               = $comply::scanner_source,
  Optional[Stdlib::Filesource] $scanner_source_linux         = $comply::scanner_source_linux,
  Optional[Stdlib::Filesource] $scanner_source_mac           = $comply::scanner_source_mac,
  Optional[Stdlib::Filesource] $scanner_source_windows       = $comply::scanner_source_windows,
  Optional[Pattern[/\d+\.\d+\.\d+/]] $scanner_version        = $comply::scanner_version,
  Optional[String] $scanner_checksum_linux                   = $comply::scanner_checksum_linux,
  Optional[String] $scanner_checksum_mac                     = $comply::scanner_checksum_mac,
  Optional[String] $scanner_checksum_windows                 = $comply::scanner_checksum_windows,
  Boolean $use_mtls                                          = $comply::use_mtls,
  Boolean $windows_manage_wget                               = $comply::windows_manage_wget,
  Optional[String] $assessor_download_token                  = $comply::assessor_download_token,
  Optional[Hash[String[1],Hash[String[1], Integer]]] $limits = $comply::limits,
) inherits comply {
  if $facts['kernel'] == 'windows' {
    $install_path                      = 'C:/ProgramData/PuppetLabs/comply'
    $download_provider                 = 'wget'
    $download_provider_path            = ['C:/ProgramData/chocolatey/bin', 'C:/Windows/System32']
    $download_provider_outfile_option  = '-O'
    $download_provider_insecure_option = '--no-check-certificate'
    $scanner_location                  = $scanner_source_windows
    $scanner_checksum                  = $scanner_checksum_windows
  } else {
    $install_path                      = '/opt/puppetlabs/comply'
    $download_provider                 = 'curl'
    $download_provider_path            = ['/opt/puppetlabs/puppet/bin', '/usr/bin']
    $download_provider_outfile_option  = '-o'
    $download_provider_insecure_option = '--insecure'
    if $facts['kernel'] == 'darwin' {
      $scanner_location = $scanner_source_mac
      $scanner_checksum = $scanner_checksum_mac
    } else {
      $scanner_location = $scanner_source_linux
      $scanner_checksum = $scanner_checksum_linux
    }
  }
  $download_filename = "Assessor-CLI-${scanner_version}.zip"

  if $manage_chocolatey and $facts['kernel'] == 'windows' {
    include chocolatey
  }

  if $manage_java {
    notice('Setting manage_java is DEPRECATED and will be removed in a subsequent version')
  }

  if $facts['kernel'] == 'windows' {
    if $windows_manage_wget {
      package { 'wget':
        ensure          => latest,
        provider        => 'chocolatey',
        install_options => ['-y'],
      }
    }
  }

  if $linux_manage_unzip and $facts['kernel'] == 'Linux' {
    ensure_packages('unzip', {
        ensure => 'installed'
    })
  }

  if $win_manage_7zip and $facts['kernel'] == 'windows' {
    ensure_packages('7zip', {
        ensure   => 'installed',
        provider => 'chocolatey',
    })
  }

  if $use_mtls and $allow_insecure {
    fail('Cannot use mTLS and allow insecure together.')
  }

  $download_options = $use_mtls ? {
    false => $allow_insecure ? {
      true  => [$download_provider_insecure_option],
      false => [],
    },
    true  => $facts['kernel'] ? {
      /(Darwin|Linux)/ => [
        '--cert',
        "${facts['ssldir']}/certs/${trusted['certname']}.pem",
        '--key',
        "${facts['ssldir']}/private_keys/${trusted['certname']}.pem",
        '--cacert',
        "${facts['ssldir']}/certs/ca.pem"
      ],
      'windows'        => [
        '--certificate',
        "${facts['ssldir']}/certs/${trusted['certname']}.pem",
        '--private-key',
        "${facts['ssldir']}/private_keys/${trusted['certname']}.pem",
        '--ca-certificate',
        "${facts['ssldir']}/certs/ca.pem"
      ],
      default          => undef,
    }
  }

  $auth_header = $assessor_download_token == '' ? {
    true  => [],
    false => ['--header', "Authorization: ${assessor_download_token}"],
  }

  file {
    default:
      ensure => directory,
      before => Exec[$download_filename],
      ;
    $install_path: ;
    "${install_path}/assessors": ;
    "${install_path}/assessors/${scanner_version}": ;
    "${install_path}/tmp": ;
  }

  if (versioncmp($facts['puppetversion'], '6.24.0') < 0) or
  ((versioncmp($facts['puppetversion'], '7.9.0') < 0) and (versioncmp($facts['puppetversion'], '7.0.0') >= 0)) {
    $download_options_str = join($download_options, ' ')
    $auth_header_str = join($auth_header, ' ')
    $download_command = "${download_provider} ${download_options_str} ${auth_header_str} ${download_provider_outfile_option} ${download_filename} ${scanner_location}"
  } else {
    $download_command = flatten([
        $download_provider,
        $download_options,
        $auth_header,
        $download_provider_outfile_option,
        $download_filename,
        $scanner_location,
    ])
  }

  exec { $download_filename:
    command => $download_command,
    creates => "${install_path}/assessors/${scanner_version}/Assessor-CLI/Assessor-CLI.jar",
    cwd     => "${install_path}/tmp",
    path    => $download_provider_path,
    before  => Archive[$download_filename],
  }

  archive { $download_filename:
    creates         => "${install_path}/assessors/${scanner_version}/Assessor-CLI/Assessor-CLI.jar",
    path            => "${install_path}/tmp/${download_filename}",
    source          => "${install_path}/tmp/${download_filename}",
    extract         => true,
    extract_path    => "${install_path}/assessors/${scanner_version}",
    checksum_verify => true,
    checksum_type   => 'sha256',
    checksum        => $scanner_checksum,
    cleanup         => false,
  }

  file {
    default:
      require => Archive[$download_filename],
      ;
    "${install_path}/Assessor-CLI":
      ensure => link,
      force  => true,
      target => "${install_path}/assessors/${scanner_version}/Assessor-CLI",
      ;
    "${install_path}/assessors/${scanner_version}/Assessor-CLI/Assessor-CLI.jar":
      mode => '0755',
      ;
    "${install_path}/assessors/${scanner_version}/Assessor-CLI/jre/bin/java":
      mode => '0755',
      ;
  }

  if $facts['kernel'] != 'windows' {
    if $limits {
      file { "${install_path}/assessors/${scanner_version}/Assessor-CLI/LIMITS":
        ensure  => file,
        content => $limits.to_json,
        mode    => '0644',
      }
    } else {
      file { "${install_path}/assessors/${scanner_version}/Assessor-CLI/LIMITS":
        ensure  => absent,
      }
    }
  }
}
